#treegulp

This is treegulp. treegulp is a tool for organizing your gulp files.

###What does it do?

treegulp lets you describe your gulp tasks in a tree hierarchy format.
If a gulp file has any task that has a dependency on another task, then you had to list all of the dependencies for each task.
treegulp removes this minor inconvenience and figures out dependencies for you.
Consider this gulpfile:

    var gulp = require('gulp');

    gulp.task('alpha', function() {
      console.log('alpha');
    });

    gulp.task('alpha-alpha', ['alpha'], function() {
      console.log('alpha-alpha');
    });

    gulp.task('alpha-beta', ['alpha'], function() {
      console.log('alpha-beta');
    });

    gulp.task('alpha-beta-alpha', ['alpha-beta'], function() { 
      console.log('alpha-beta-alpha');
    });

    gulp.task('beta', function() {
      console.log('beta');
    });

    gulp.task('beta-alpha', ['beta'], function() {
      console.log('beta-alpha');
    });

    gulp.task('beta-alpha-alpha', ['beta-alpha'], function() {
      console.log('beta-alpha-alpha');
    });

    gulp.task('beta-alpha-alpha-alpha', ['beta-alpha-alpha'], function() {
      console.log('beta-alpha-alpha-alpha');
    });

    gulp.task('default', ['alpha', 'beta']);

We can rewrite this to be a little bit cleaner (subjectively) using treegulp.
Whenever task A has task B as a dependency, we can just nest task B inside task A.
Here is how the gulpfile will look with treegulp:

    var treegulp = require('treegulp');

    treegulp('default
      treegulp('alpha',
        function() {
          console.log('alpha');
        },
        treegulp('alpha-alpha',
          function() {
            console.log('alpha-alpha');
          }
        ),
        treegulp('alpha-beta',
          function() {
            console.log('alpha-beta');
          },
          treegulp('alpha-beta-alpha',
            function() {
              console.log('alpha-beta-alpha');
            }
          )
        )
      ),
      treegulp('beta',
        function() {
          console.log('beta');
        },
        treegulp('beta-alpha',
          function() {
            console.log('beta-alpha');
          },
          treegulp('beta-alpha-alpha',
            function() {
              console.log('beta-alpha-alpha');
            },
            treegulp('beta-alpha-alpha-alpha',
              function() {
                console.log('beta-alpha-alpha-alpha');
              }
            )
          )
        )
      )
    );

###How does it work?

The treegulp module provides the treegulp function.
The treegulp function accepts a set of any number of arguments.
The order of the arguments doesn't matter.
The treegulp function collects all of its arguments and puts them into one of three lists:

1. If the argument is a string, it is put into the names list and may be used as a name for the created gulp task.

2. If the argument is an object, it is assumed (for now) to be a treegulp object that another call to treegulp returned.
It is thus added to the dependency list. This allows task nesting.

3. If the argument is a method, it is put into the list of methods.
Each of these methods is run (in no particular order) whenever the created task runs.

If any of the arguments is an array, it puts each element of the array into one of the three lists using the same logic.
This means that these two code snippets are equivalent:

    treegulp('alpha', 'do-alpha', callbackA, callbackB, callbackC);

    treegulp(['alpha', 'do-alpha'], [['callbackA', ['callbackB', 'callbackC']]]);
    
The treegulp function returns an object that contains the three lists, named 'names', 'dependencies', and 'methods'.
    
###What if you want to do it the other way around?

Coming soon!