var gulp = require('gulp');
var crypto = require('crypto');

var treegulp = function() {
  var i,
    args = Array.prototype.slice.call(arguments), // Make a real Array.
    result = {};
  result.names = [];
  result.dependencies = [];
  result.methods = [];
  
  // Have to loop primitively because 'args' may grow in the loop.
  for (i = 0; i < args.length; ++i) {
    var argument = args[i];
    
    // If the argument is a string, treat it as a name.
    if (typeof argument === 'string') {
      result.names.push(argument);
    }
    // If it's a function, use it as a callback method.
    else if (typeof argument === 'function') {
      result.methods.push(argument);
    }
    // If it's an array, add each element into args to process later.
    else if (Array.isArray(argument)) {
      args = args.concat(argument);
    }
    // Objects are assumed to be of the form returned by this function.
    else if (typeof argument === 'object') {
      result.dependencies.push(argument);
    }
  }

  // These are the dependencies that will actually be registered with gulp.
  var dependenciesToRegister = result.dependencies;
  
  // This is the callback that will actually be registered for each task name.
  var methodToRegister;
  
  // If there are no specified methods, then create gulp tasks with no
  // callback that just act as a substitute for the dependencies.
  // This will happen by default.

  // If there is exactly one method, use it as the callback for each task.
  if (result.methods.length === 1) {
    methodToRegister = result.methods[0];
  }
  // If there are multiple methods, make each task be a substitute for the
  // set of "anonymous" tasks created from the set of supplied methods.
  else if (result.methods.length > 1) {
    dependenciesToRegister = [];
    result.methods.forEach(function(method) {
      // Create a random task name.
      var anonymousTaskName = crypto.createHash('md5');
      dependenciesToRegister.push(anonymousTaskName);
      
      // Register the anonymous task.
      // The given dependencies are dependencies for each "anonymous" task.
      gulp.task(anonymousTaskName, result.dependencies, method);
    });
  }
  
  // If there are no specified names, come up with a single random name.
  if (result.names.length === 0) {
    result.names[0] = crypto.createHash('md5');
  }
  
  // Now actually register the tasks.
  result.names.forEach(function(name) {
    gulp.task(name, dependenciesToRegister, methodToRegister);
  });
  
  return result;
}

// Make 'treegulp.task' an alias for 'treegulp'.
treegulp.task = treegulp;

// Export treegulp as a module.
module.exports = treegulp;
